@@ -7,7 +7,7 @@ maximal number of threads they can use.
 # License: BSD 3-Clause
 
 # The code to introspect dynamically loaded libraries on POSIX systems is
-# adapted from code by Intel developper @anton-malakhov available at
+# adapted from code by Intel developer @anton-malakhov available at
 # https://github.com/IntelPython/smp (Copyright (c) 2017, Intel Corporation)
 # and also published under the BSD 3-Clause license
 import os
@@ -21,7 +21,7 @@ from abc import ABC, abstractmethod
 from functools import lru_cache
 from contextlib import ContextDecorator
 
-__version__ = "3.0.0"
+__version__ = "3.1.0"
 __all__ = ["threadpool_limits", "threadpool_info", "ThreadpoolController"]
 
 
@@ -61,9 +61,11 @@ except AttributeError:
 
 
 # List of the supported libraries. The items are indexed by the name of the
-# class to instanciate to create the library controller objects. The items hold
+# class to instantiate to create the library controller objects. The items hold
 # the possible prefixes of loaded shared objects, the name of the internal_api
-# to call and the name of the user_api.
+# to call, the name of the user_api and potentially some symbols that the library is
+# expected to have (this is necessary to distinguish between the blas implementations
+# when they are all renamed "libblas.dll" on conda-forge on windows).
 _SUPPORTED_LIBRARIES = {
     "OpenMPController": {
         "user_api": "openmp",
@@ -73,28 +75,33 @@ _SUPPORTED_LIBRARIES = {
     "OpenBLASController": {
         "user_api": "blas",
         "internal_api": "openblas",
-        "filename_prefixes": ("libopenblas",),
+        "filename_prefixes": ("libopenblas", "libblas"),
+        "check_symbols": ("openblas_get_num_threads", "openblas_get_num_threads64_"),
     },
     "MKLController": {
         "user_api": "blas",
         "internal_api": "mkl",
-        "filename_prefixes": ("libmkl_rt", "mkl_rt"),
+        "filename_prefixes": ("libmkl_rt", "mkl_rt", "libblas"),
+        "check_symbols": ("MKL_Get_Max_Threads",),
     },
     "BLISController": {
         "user_api": "blas",
         "internal_api": "blis",
-        "filename_prefixes": ("libblis",),
+        "filename_prefixes": ("libblis", "libblas"),
+        "check_symbols": ("bli_thread_get_num_threads",),
     },
 }
 
 # Helpers for the doc and test names
 _ALL_USER_APIS = list(set(lib["user_api"] for lib in _SUPPORTED_LIBRARIES.values()))
 _ALL_INTERNAL_APIS = [lib["internal_api"] for lib in _SUPPORTED_LIBRARIES.values()]
-_ALL_PREFIXES = [
-    prefix
-    for lib in _SUPPORTED_LIBRARIES.values()
-    for prefix in lib["filename_prefixes"]
-]
+_ALL_PREFIXES = list(
+    set(
+        prefix
+        for lib in _SUPPORTED_LIBRARIES.values()
+        for prefix in lib["filename_prefixes"]
+    )
+)
 _ALL_BLAS_LIBRARIES = [
     lib["internal_api"]
     for lib in _SUPPORTED_LIBRARIES.values()
@@ -151,10 +158,10 @@ class _ThreadpoolLimiter:
     """
 
     def __init__(self, controller, *, limits=None, user_api=None):
+        self._controller = controller
         self._limits, self._user_api, self._prefixes = self._check_params(
             limits, user_api
         )
-        self._controller = controller
         self._original_info = self._controller.info()
         self._set_threadpool_limits()
 
@@ -219,6 +226,13 @@ class _ThreadpoolLimiter:
 
     def _check_params(self, limits, user_api):
         """Suitable values for the _limits, _user_api and _prefixes attributes"""
+
+        if isinstance(limits, str) and limits == "sequential_blas_under_openmp":
+            (
+                limits,
+                user_api,
+            ) = self._controller._get_params_for_sequential_blas_under_openmp().values()
+
         if limits is None or isinstance(limits, int):
             if user_api is None:
                 user_api = _ALL_USER_APIS
@@ -250,8 +264,8 @@ class _ThreadpoolLimiter:
 
             if not isinstance(limits, dict):
                 raise TypeError(
-                    "limits must either be an int, a list or a "
-                    f"dict. Got {type(limits)} instead"
+                    "limits must either be an int, a list, a dict, or "
+                    f"'sequential_blas_under_openmp'. Got {type(limits)} instead"
                 )
 
             # With a dictionary, can set both specific limit for given
@@ -326,7 +340,7 @@ class threadpool_limits(_ThreadpoolLimiter):
 
     Parameters
     ----------
-    limits : int, dict or None (default=None)
+    limits : int, dict, 'sequential_blas_under_openmp' or None (default=None)
         The maximal number of threads that can be used in thread pools
 
         - If int, sets the maximum number of threads to `limits` for each
@@ -336,6 +350,11 @@ class threadpool_limits(_ThreadpoolLimiter):
           custom maximum number of threads for each `key` which can be either a
           `user_api` or a `prefix` for a specific library.
 
+        - If 'sequential_blas_under_openmp', it will chose the appropriate `limits`
+          and `user_api` parameters for the specific use case of sequential BLAS
+          calls within an OpenMP parallel region. The `user_api` parameter is
+          ignored.
+
         - If None, this function does not do anything.
 
     user_api : {USER_APIS} or None (default=None)
@@ -421,6 +440,18 @@ class ThreadpoolController:
 
         return ThreadpoolController._from_controllers(lib_controllers)
 
+    def _get_params_for_sequential_blas_under_openmp(self):
+        """Return appropriate params to use for a sequential BLAS call in an OpenMP loop
+
+        This function takes into account the unexpected behavior of OpenBLAS with the
+        OpenMP threading layer.
+        """
+        if self.select(
+            internal_api="openblas", threading_layer="openmp"
+        ).lib_controllers:
+            return {"limits": None, "user_api": None}
+        return {"limits": 1, "user_api": "blas"}
+
     @_format_docstring(
         USER_APIS=", ".join('"{}"'.format(api) for api in _ALL_USER_APIS),
         BLAS_LIBS=", ".join(_ALL_BLAS_LIBRARIES),
@@ -444,7 +475,7 @@ class ThreadpoolController:
 
         Parameters
         ----------
-        limits : int, dict or None (default=None)
+        limits : int, dict, 'sequential_blas_under_openmp' or None (default=None)
             The maximal number of threads that can be used in thread pools
 
             - If int, sets the maximum number of threads to `limits` for each
@@ -454,6 +485,11 @@ class ThreadpoolController:
               custom maximum number of threads for each `key` which can be either a
               `user_api` or a `prefix` for a specific library.
 
+            - If 'sequential_blas_under_openmp', it will chose the appropriate `limits`
+              and `user_api` parameters for the specific use case of sequential BLAS
+              calls within an OpenMP parallel region. The `user_api` parameter is
+              ignored.
+
             - If None, this function does not do anything.
 
         user_api : {USER_APIS} or None (default=None)
@@ -526,7 +562,7 @@ class ThreadpoolController:
         """Loop through loaded libraries and return binders on supported ones
 
         This function is expected to work on POSIX system only.
-        This code is adapted from code by Intel developper @anton-malakhov
+        This code is adapted from code by Intel developer @anton-malakhov
         available at https://github.com/IntelPython/smp
 
         Copyright (c) 2017, Intel Corporation published under the BSD 3-Clause
@@ -660,6 +696,26 @@ class ThreadpoolController:
             if prefix is None:
                 continue
 
+            # workaround for BLAS libraries packaged by conda-forge on windows, which
+            # are all renamed "libblas.dll". We thus have to check to which BLAS
+            # implementation it actually corresponds looking for implementation
+            # specific symbols.
+            if prefix == "libblas":
+                if filename.endswith(".dll"):
+                    libblas = ctypes.CDLL(filepath, _RTLD_NOLOAD)
+                    if not any(
+                        hasattr(libblas, func)
+                        for func in candidate_lib["check_symbols"]
+                    ):
+                        continue
+                else:
+                    # We ignore libblas on other platforms than windows because there
+                    # might be a libblas dso comming with openblas for instance that
+                    # can't be used to instantiate a pertinent LibController (many
+                    # symbols are missing) and would create confusion by making a
+                    # duplicate entry in threadpool_info.
+                    continue
+
             # filename matches a prefix. Create and store the library
             # controller.
             user_api = candidate_lib["user_api"]
